==> apt-repo-0.1.tar.gz: apt-repo-0.1/apt_repo/__init__.py <==
import bz2
import gzip
import lzma
import os
import re
import urllib.error
import urllib.request as request


def download_raw(url):
    return request.urlopen(url).read()


def download(url):
    return download_raw(url).decode('utf-8')


decompress = {
    '': lambda c: c,
    '.xz': lambda c: lzma.decompress(c),
    '.gz': lambda c: gzip.decompress(c),
    '.bzip2': lambda c: bz2.decompress(c)
}


def download_compressed(base_url):
    for suffix, method in decompress.items():
        url = base_url + suffix

        try:
            req = request.urlopen(url)
        except urllib.error.URLError:
            continue

        return method(req.read()).decode('utf-8')


def get_value(content, key):
    pattern = key + ': (.*)\n'
    match = re.search(pattern, content)
    try:
        return match.group(1)
    except AttributeError:
        raise KeyError(content, key)


class ReleaseFile:
    """Class that represents a Release file"""
    def __init__(self, content):
        self.__content = content.strip()

    @property
    def origin(self):
        return get_value(self.__content, 'Origin')

    @property
    def label(self):
        return get_value(self.__content, 'Label')

    @property
    def suite(self):
        return get_value(self.__content, 'Suite')

    @property
    def version(self):
        return get_value(self.__content, 'Version')

    @property
    def codename(self):
        return get_value(self.__content, 'Codename')

    @property
    def date(self):
        return get_value(self.__content, 'Date')

    @property
    def architectures(self):
        return get_value(self.__content, 'Architectures').split()

    @property
    def components(self):
        return get_value(self.__content, 'Components').split()

    @property
    def description(self):
        return get_value(self.__content, 'Description')


class PackagesFile:
    """"""
    def __init__(self, content):
        self.__content = content.strip()

    @property
    def packages(self):
        packages = []
        for package_content in self.__content.split('\n\n'):
            if not package_content:
                continue

            packages.append(BinaryPackage(package_content))

        return packages


class BinaryPackage:
    def __init__(self, content):
        self.__content = content.strip()
        
    @property
    def package(self):
        return get_value(self.__content, 'Package')

    @property
    def version(self):
        return get_value(self.__content, 'Version')


class APTRepository:
    def __init__(self, url, dist, components):
        self.__url = url
        self.__dist = dist
        self.__components = components

    @property
    def components(self):
        return self.__components

    @property
    def all_components(self):
        return self.release_file.components

    @property
    def release_file(self):
        url = os.path.join(
            self.__url,
            'dists',
            self.__dist,
            'Release'
        )

        release_content = download(url)

        return ReleaseFile(release_content)

    @property
    def packages(self, arch='amd64'):
        packages = []
        for component in self.__components:
            packages.extend(self.get_binary_packages_by_component(component, arch))

        return packages

    def get_binary_packages_by_component(self, component, arch='amd64'):
        url = os.path.join(
            self.__url,
            'dists',
            self.__dist,
            component,
            'binary-' + arch,
            'Packages'
        )

        packages_file = download_compressed(url)

        return PackagesFile(packages_file).packages

    def get_packages_by_name(self, name):
        all_binary_packages = self.packages
        packages = []

        for package in all_binary_packages:
            if package.package == name:
                packages.append(package)

        return packages


class APTSources:
    def __init__(self, repositories):
        self.__repositories = repositories

    @property
    def packages(self):
        packages = []

        for repo in self.__repositories:
            packages.extend(repo.packages)

        return packages

    def get_packages_by_name(self, name):
        packages = []

        for repo in self.__repositories:
            packages.extend(repo.get_packages_by_name(name))

        return packages

==> apt-repo-0.2.tar.gz: apt-repo-0.2/apt_repo/__init__.py <==
import bz2
import gzip
import lzma
import os
import re
import urllib.error
import urllib.request as request


def download_raw(url):
    return request.urlopen(url).read()


def download(url):
    return download_raw(url).decode('utf-8')


decompress = {
    '': lambda c: c,
    '.xz': lambda c: lzma.decompress(c),
    '.gz': lambda c: gzip.decompress(c),
    '.bzip2': lambda c: bz2.decompress(c)
}


def download_compressed(base_url):
    for suffix, method in decompress.items():
        url = base_url + suffix

        try:
            req = request.urlopen(url)
        except urllib.error.URLError:
            continue

        return method(req.read()).decode('utf-8')


def get_value(content, key):
    pattern = key + ': (.*)\n'
    match = re.search(pattern, content)
    try:
        return match.group(1)
    except AttributeError:
        raise KeyError(content, key)


class ReleaseFile:
    """Class that represents a Release file"""
    def __init__(self, content):
        self.__content = content.strip()

    @property
    def origin(self):
        return get_value(self.__content, 'Origin')

    @property
    def label(self):
        return get_value(self.__content, 'Label')

    @property
    def suite(self):
        return get_value(self.__content, 'Suite')

    @property
    def version(self):
        return get_value(self.__content, 'Version')

    @property
    def codename(self):
        return get_value(self.__content, 'Codename')

    @property
    def date(self):
        return get_value(self.__content, 'Date')

    @property
    def architectures(self):
        return get_value(self.__content, 'Architectures').split()

    @property
    def components(self):
        return get_value(self.__content, 'Components').split()

    @property
    def description(self):
        return get_value(self.__content, 'Description')


class PackagesFile:
    """"""
    def __init__(self, content):
        self.__content = content.strip()

    @property
    def packages(self):
        packages = []
        for package_content in self.__content.split('\n\n'):
            if not package_content:
                continue

            packages.append(BinaryPackage(package_content))

        return packages


class BinaryPackage:
    def __init__(self, content):
        self.__content = content.strip()
        
    @property
    def package(self):
        return get_value(self.__content, 'Package')

    @property
    def version(self):
        return get_value(self.__content, 'Version')

    @property
    def filename(self):
        return get_value(self.__content, 'Filename')


class APTRepository:
    def __init__(self, url, dist, components):
        self.__url = url
        self.__dist = dist
        self.__components = components

    def __getitem__(self, item):
        return self.get_packages_by_name(item)

    @staticmethod
    def from_sources_list_entry(entry):
        split_entry = entry.split()

        url = split_entry[1]
        dist = split_entry[2]
        components = split_entry[3:]

        return APTRepository(url, dist, components)

    @property
    def components(self):
        return self.__components

    @property
    def all_components(self):
        return self.release_file.components

    @property
    def release_file(self):
        url = os.path.join(
            self.__url,
            'dists',
            self.__dist,
            'Release'
        )

        release_content = download(url)

        return ReleaseFile(release_content)

    @property
    def packages(self, arch='amd64'):
        packages = []
        for component in self.__components:
            packages.extend(self.get_binary_packages_by_component(component, arch))

        return packages

    def get_binary_packages_by_component(self, component, arch='amd64'):
        url = os.path.join(
            self.__url,
            'dists',
            self.__dist,
            component,
            'binary-' + arch,
            'Packages'
        )

        packages_file = download_compressed(url)

        return PackagesFile(packages_file).packages

    def get_package(self, name, version):
        for package in self.packages:
            if package.package == name and package.version == version:
                return package

        raise KeyError(name, version)

    def get_package_url(self, name, version):
        package = self.get_package(name, version)

        return os.path.join(self.__url, package.filename)

    def get_packages_by_name(self, name):
        packages = []

        for package in self.packages:
            if package.package == name:
                packages.append(package)

        return packages


class APTSources:
    def __init__(self, repositories):
        self.__repositories = repositories

    def __getitem__(self, item):
        return self.get_packages_by_name(item)

    @property
    def packages(self):
        packages = []

        for repo in self.__repositories:
            packages.extend(repo.packages)

        return packages

    def get_package(self, name, version):
        for repo in self.__repositories:
            try:
                return repo.get_package(name, version)
            except KeyError:
                pass

        raise KeyError(name, version)

    def get_package_url(self, name, version):
        for repo in self.__repositories:
            try:
                return repo.get_package_url(name, version)
            except KeyError:
                pass

        raise KeyError(name, version)

    def get_packages_by_name(self, name):
        packages = []

        for repo in self.__repositories:
            packages.extend(repo.get_packages_by_name(name))

        return packages
